//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Zobrazovanie cislel 0-99 na dvoch 7-segmentovkach
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

// Definicia konstant pre zobrazovanie znakov na 7-segmentovke
char znaky[16]={0x3f,    // znak '0'
                0x06,    // znak '1'
                0x5b,    // znak '2'
                0x4f,    // znak '3'
                0x66,    // znak '4'
                0x6d,    // znak '5'
                0x7d,    // znak '6'
                0x07,    // znak '7'
                0x7f,    // znak '8'
                0x6f,    // znak '9'
                0x77,    // znak 'A'
                0x7c,    // znak 'b'
                0x58,    // znak 'c'
                0x5e,    // znak 'd'
                0x79,    // znak 'E'
                0x71};   // znak 'F'

//  *******************************************************
//  Kratke cakanie (cca 450 us)
//  *******************************************************
void cakaj(void)
{ unsigned int cas;
 
  for(cas=0; cas < MCK/20000; cas++);
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ int index;

  //******************* Uvodna inicializacia *******************
  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = 0xffff;            // piny A0-A15 su vystupne
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = S2 | 0xffff;

  //******************* Hlavna cast programu *******************
  // zhasnutie 7-segmentoviek
  AT91C_BASE_PIOA->PIO_SODR = 0xffff;
  // zobrazenie '0'
  AT91C_BASE_PIOA->PIO_CODR = znaky[0] << 8;

  index=0;
  // nekonecna slucka
  while(1)
  { // cakanie na stlacenie S2  
    while(AT91C_BASE_PIOA->PIO_PDSR & S2)     // kontrola, ci je S2 v log.1
      cakaj();                                // cakanie kvoli potlaceniu zakmitov
    // cakanie na pustenie S2
    while(!(AT91C_BASE_PIOA->PIO_PDSR & S2))  // kontrola, ci je S2 v log.0
      cakaj();                                // cakanie kvoli potlaceniu zakmitov

    // zabezpecenie cyklenia od 0 do 99
    index++;
    if (index > 99)
      index=0;

    // zhasnutie 7segmentoviek
    AT91C_BASE_PIOA->PIO_SODR = 0xffff;

    // zobrazenie cisla
    if (index<10)
      AT91C_BASE_PIOA->PIO_CODR = znaky[index] << 8;                   // jednomiestne cislo
    else
      AT91C_BASE_PIOA->PIO_CODR = znaky[index/10] | (znaky[index%10] << 8);  // dvojmiestne cislo
  }
}
